@@ -2,7 +2,7 @@ typedef struct {
 	int	r,g,b;
 } VGACOLOR;
 
-extern PIXEL	*vid_lines[];
+extern void	*vid_lines[];
 
 int	VID_outputInit(void);
 void	VID_outputImage(void);
