@@ -124,7 +124,7 @@ BOOL InitApplication(HINSTANCE hInstance,int nCmdShow)
 	
 	if (!hWnd) return FALSE;
 
-	hf = fopen("bootlog.txt","w");
+	hf = fopen("BOOTLOG.TXT","wt");
 	*stdout = *hf;
 	i = setvbuf( stdout, NULL, _IONBF, 0 );
 
@@ -303,30 +303,33 @@ typedef struct tagDlgCtrls {
 } DlgCtrls;
 
 DlgCtrls dlgctrls[] = {
-	{ 420, FALSE, 2, "Trace unchecked", "trace"},
+	// add sound checkbox (mpc)
+	{ 419, FALSE, 1, "Sound unchecked", "nosound"},
+
+	{ 420, FALSE, 3, "Trace unchecked", "trace"},
 	
-	{ 421, FALSE, 1, "1MB RAM", "ram 1"},
-	{ 421, TRUE, 1, "2MB RAM", "ram 2"},
-	{ 421, FALSE, 1, "3MB RAM", "ram 3"},
-	{ 421, FALSE, 1, "4MB RAM", "ram 4"},
-	{ 421, FALSE, 1, "5MB RAM", "ram 5"},
-	{ 421, FALSE, 1, "6MB RAM", "ram 6"},
-	{ 421, FALSE, 1, "7MB RAM", "ram 7"},
-	{ 421, FALSE, 1, "8MB RAM", "ram 8"},
+	{ 421, FALSE, 2, "1MB RAM", "ram 1"},
+	{ 421, TRUE, 2, "2MB RAM", "ram 2"},
+	{ 421, FALSE, 2, "3MB RAM", "ram 3"},
+	{ 421, FALSE, 2, "4MB RAM", "ram 4"},
+	{ 421, FALSE, 2, "5MB RAM", "ram 5"},
+	{ 421, FALSE, 2, "6MB RAM", "ram 6"},
+	{ 421, FALSE, 2, "7MB RAM", "ram 7"},
+	{ 421, FALSE, 2, "8MB RAM", "ram 8"},
 
-	{ 422, TRUE, 3, "C\\XGS", ""},
+	{ 422, TRUE, 4, "C:\\XGS", ""},
 
-	{ 423, TRUE, 4, "<None>", "s5d1"},
-	{ 424, TRUE, 4, "<None>", "s5d2"},
-	{ 425, TRUE, 4, "<None>", "s6d1"},
-	{ 426, TRUE, 4, "<None>", "s6d2"},
-	{ 427, TRUE, 4, "<None>", "smpt0"},
-	{ 428, TRUE, 4, "<None>", "smpt1"},
-	{ 429, TRUE, 4, "<None>", "smpt2"},
-	{ 430, TRUE, 4, "<None>", "smpt3"},
-	{ 431, TRUE, 4, "<None>", "smpt4"},
-	{ 432, TRUE, 4, "<None>", "smpt5"},
-	{ 433, TRUE, 4, "<None>", "smpt6"},
+	{ 423, TRUE, 5, "<None>", "s5d1"},
+	{ 424, TRUE, 5, "<None>", "s5d2"},
+	{ 425, TRUE, 5, "<None>", "s6d1"},
+	{ 426, TRUE, 5, "<None>", "s6d2"},
+	{ 427, TRUE, 5, "<None>", "smpt0"},
+	{ 428, TRUE, 5, "<None>", "smpt1"},
+	{ 429, TRUE, 5, "<None>", "smpt2"},
+	{ 430, TRUE, 5, "<None>", "smpt3"},
+	{ 431, TRUE, 5, "<None>", "smpt4"},
+	{ 432, TRUE, 5, "<None>", "smpt5"},
+	{ 433, TRUE, 5, "<None>", "smpt6"},
 
     { 0, 0}  // End Of List
 };
@@ -359,14 +362,22 @@ BOOL APIENTRY CLDlgProc (HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
 
 			strcpy(ImagePath,XGS_DIR);
 			strcpy(tempstring2, XGS_DIR);
-			strcat(tempstring2, "\\xgs.ini");
-			fp1 = fopen(tempstring2,"r");
+			strcat(tempstring2, "\\XGS.INI");
+			fp1 = fopen(tempstring2,"rt");
             iCtrl = i = 0;
 			memval = -1;
             while (dlgctrls[i].ctrlId) {
 				iCtrl = dlgctrls[i].ctrlId;
 				switch (dlgctrls[i].type) {
 					case 1:
+						SendDlgItemMessage (hdlg, iCtrl, BM_SETCHECK, dlgctrls[i].def, 0);
+						if (fp1 != NULL) {
+							fscanf(fp1, "%d", &j);
+							if (j) SendDlgItemMessage (hdlg, iCtrl, BM_SETCHECK, 1, 0);
+							while ((f = getc(fp1)) != '\n');
+						}
+						break;
+					case 2:
 						index = SendDlgItemMessage (hdlg, iCtrl, CB_ADDSTRING, 0, (DWORD)(LPSTR)dlgctrls[i].str);
 						SendDlgItemMessage (hdlg, iCtrl, CB_SETITEMDATA, index, i);
 						if ((dlgctrls[i].def) && (fp1 == NULL)) {
@@ -380,7 +391,7 @@ BOOL APIENTRY CLDlgProc (HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
 							if (memval == i) SendDlgItemMessage (hdlg, dlgctrls[i].ctrlId, CB_SETCURSEL, index, 0);
 						}
 						break;
-					case 2:
+					case 3:
 						SendDlgItemMessage (hdlg, iCtrl, BM_SETCHECK, dlgctrls[i].def, 0);
 						if (fp1 != NULL) {
 							fscanf(fp1, "%d", &j);
@@ -388,7 +399,7 @@ BOOL APIENTRY CLDlgProc (HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
 							while ((f = getc(fp1)) != '\n');
 						}
 						break;
-					case 3:
+					case 4:
 						SendDlgItemMessage (hdlg, iCtrl, WM_SETTEXT, 0, (DWORD)(LPSTR)dlgctrls[i].str);
 						if (fp1 != NULL) {
 							j = 0;
@@ -399,9 +410,10 @@ BOOL APIENTRY CLDlgProc (HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
 							while ((f = getc(fp1)) != '\n');
 						}
 						break;
-					case 4:
+					case 5:
 						UpdateMyControl(i, iCtrl, hdlg, ImagePath, &fp1);
 						break;
+
 				}
                 i++;
             }
@@ -433,6 +445,21 @@ BOOL APIENTRY CLDlgProc (HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
                                 iCtrl = dlgctrls[i].ctrlId;
 								switch (dlgctrls[i].type) {
 									case 1:
+										if (!SendDlgItemMessage (hdlg, iCtrl, BM_GETCHECK, 0, 0)) {
+											strcpy(tempstring, dlgctrls[i].opt);
+											tokenPtr = strtok(tempstring, " ");
+											wsprintf ((LPSTR)cmd, "-");
+											cmd++;
+											while (tokenPtr != NULL) {
+												wsprintf ((LPSTR)cmd, "%s", (LPSTR)tokenPtr);
+												cmd += strlen(cmd);
+												cmd[0] = 0;
+												argv[++argc] = ++cmd;
+												tokenPtr = strtok(NULL, " ");
+											}
+										}
+										break;
+									case 2:
 										index = SendDlgItemMessage(hdlg, iCtrl, CB_GETCURSEL, 0, 0);
 										if (index) {
 											item = SendDlgItemMessage (hdlg, iCtrl, CB_GETITEMDATA, index, 0);
@@ -449,7 +476,7 @@ BOOL APIENTRY CLDlgProc (HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
 											}
 										}
 										break;
-									case 2:
+									case 3:
 										if (SendDlgItemMessage (hdlg, iCtrl, BM_GETCHECK, 0, 0)) {
 											strcpy(tempstring, dlgctrls[i].opt);
 											tokenPtr = strtok(tempstring, " ");
@@ -464,7 +491,7 @@ BOOL APIENTRY CLDlgProc (HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
 											}
 										}
 										break;
-									case 3:
+									case 4:
 /*										if (SendDlgItemMessage (hdlg, iCtrl, WM_GETTEXT, (DWORD)511, (DWORD)tempstring)) {
 											tokenPtr = strtok(tempstring, " ");
 											wsprintf ((LPSTR)cmd, "-%s", (LPSTR)dlgctrls[i].opt);
@@ -480,7 +507,7 @@ BOOL APIENTRY CLDlgProc (HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
 											} 
 										} */
 										break;
-									case 4:
+									case 5:
 										if (SendDlgItemMessage (hdlg, iCtrl, WM_GETTEXT, (DWORD)511, (DWORD)tempstring2)) {
 											if (tempstring2[0] != '<') {
 												tokenPtr = strtok(tempstring2, "-");
@@ -527,28 +554,34 @@ BOOL APIENTRY CLDlgProc (HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
 					iCtrl = i = 0;
 					
 					strcpy(tempstring2, XGS_DIR);
-					strcat(tempstring2, "\\xgs.ini");
-					if ((fp1 = fopen(tempstring2,"w")) != NULL) {
+					strcat(tempstring2, "\\XGS.INI");
+					if ((fp1 = fopen(tempstring2,"wt")) != NULL) {
 						while (dlgctrls[i].ctrlId) {
 							if (dlgctrls[i].ctrlId != iCtrl) { // Starting a new list
 								iCtrl = dlgctrls[i].ctrlId;
 								switch (dlgctrls[i].type) {
 									case 1:
-										index = SendDlgItemMessage(hdlg, iCtrl, CB_GETCURSEL, 0, 0);
-										item = SendDlgItemMessage (hdlg, iCtrl, CB_GETITEMDATA, index, 0);
-										fprintf(fp1, "%d\n", item);
-										break;
-									case 2:
 										if (SendDlgItemMessage (hdlg, iCtrl, BM_GETCHECK, 0, 0))
 											fprintf(fp1, "1\n");
 										else
 											fprintf(fp1, "0\n");
 										break;
+									case 2:
+										index = SendDlgItemMessage(hdlg, iCtrl, CB_GETCURSEL, 0, 0);
+										item = SendDlgItemMessage (hdlg, iCtrl, CB_GETITEMDATA, index, 0);
+										fprintf(fp1, "%d\n", item);
+										break;
 									case 3:
+										if (SendDlgItemMessage (hdlg, iCtrl, BM_GETCHECK, 0, 0))
+											fprintf(fp1, "1\n");
+										else
+											fprintf(fp1, "0\n");
+										break;
+									case 4:
 										SendDlgItemMessage (hdlg, iCtrl, WM_GETTEXT, (DWORD)511, (DWORD)tempstring2);
 										fprintf(fp1, "%s?\n", tempstring2);
 										break;
-									case 4:
+									case 5:
 										SendDlgItemMessage (hdlg, iCtrl, WM_GETTEXT, (DWORD)511, (DWORD)tempstring2);
 										fprintf(fp1, "%s?\n", tempstring2);
 										break;
@@ -571,7 +604,7 @@ BOOL APIENTRY CLDlgProc (HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
 					iCtrl = i = 0;
 					while (dlgctrls[i].ctrlId) {
 						iCtrl = dlgctrls[i].ctrlId;
-						if (dlgctrls[i].type == 4) UpdateMyControl(i, iCtrl, hdlg, ImagePath, NULL);
+						if (dlgctrls[i].type == 5) UpdateMyControl(i, iCtrl, hdlg, ImagePath, NULL);
 						i++;
 					}
 					return (TRUE);
@@ -598,7 +631,7 @@ void UpdateMyControl(int i, int iCtrl, HWND hdlg, char *ImagePath, FILE **preffi
 	char f;
 
 	strcpy(SearchPath,ImagePath);
-	strcat(SearchPath,"\\*.2mg");
+	strcat(SearchPath,"\\*.2MG");
 	hSearch = FindFirstFile(SearchPath, &FileData);
 	SendDlgItemMessage (hdlg, iCtrl, CB_RESETCONTENT, 0, 0);
 	index = SendDlgItemMessage (hdlg, iCtrl, CB_ADDSTRING, 0, (DWORD)(LPSTR)dlgctrls[i].str);
