@@ -16,6 +16,8 @@
  * DirectSound driver for Windows 95 or NT 4.0 w/Service Pack 3
  * By Ian Schmidt (irsman@iag.net).
  *
+ * Driver version 1.1 (October 6, 1997).
+ *
  */
 
 #include "xgs.h"
@@ -124,9 +126,11 @@ int SND_outputInit(int rate)
 		return(0);
 	}
 
-	// ok, cool, we're ready to go!
+	// set the secondary buffer format
 
-	printf("XGS DirectSound driver v0.2 OK,\n");
+	IDirectSoundBuffer_SetFormat(lpSecB, &format);
+
+	// ok, cool, we're ready to go!
 
 	soundbuffer = 0;
 
@@ -240,6 +244,7 @@ size_t SND_outputWrite(snd_sample_struct *buffer, size_t len)
 		dskick++;
 		IDirectSoundBuffer_SetCurrentPosition(lpSecB, 0);
 		IDirectSoundBuffer_Play(lpSecB, 0, 0, DSBPLAY_LOOPING);
+//		IDirectSoundBuffer_SetFrequency(lpSecB, 34000);	// no idea why, but this makes the pitch sound right =)
 	}
 
 	return(OUTPUT_BUFFER_SIZE);
