@@ -3,9 +3,9 @@
 
 extern int	vid_out_width,vid_out_height,vid_out_x,vid_out_y;
 
-extern PIXEL	*vid_lines[];
+extern void	*vid_lines[];
 
-int		VID_outputInit(void);
+int	VID_outputInit(void);
 void	VID_outputImage(void);
 void	VID_outputShutdown();
 void	VID_outputStatus1(char *);
