@@ -79,6 +79,9 @@ void ADB_inputUpdate()
 				vid_mapped = 0;
 			break;
  		case KeyPress:
+			if (event.xkeymap.window != vid_window) {
+				break;
+			}
  			key = XLookupKeysym((XKeyEvent *) &event, 0);
  			switch(key) {
  			case XK_F5 :
@@ -208,6 +211,9 @@ void ADB_inputUpdate()
  			if (ski_input_index == ADB_INPUT_BUFFER) ski_input_index = 0;
  			break;
  		case KeyRelease:
+			if (event.xkeymap.window != vid_window) {
+				break;
+			}
  			key = XLookupKeysym((XKeyEvent *) &event, 0);
  			switch(key) {
  			case XK_Caps_Lock :
@@ -247,6 +253,9 @@ void ADB_inputUpdate()
  			}
  			break;
  		case ButtonPress:
+			if (event.xkeymap.window != vid_main_window) {
+				break;
+			}
  			if (adb_grab_mode == 1) {		/* Joystick */
  				switch(event.xbutton.button) {
 					case Button1 :	ski_modifier_reg |= 0x80;
@@ -272,6 +281,9 @@ void ADB_inputUpdate()
 			}
  			break;
  		case ButtonRelease:
+			if (event.xkeymap.window != vid_main_window) {
+				break;
+			}
  			if (adb_grab_mode == 1) {		/* Joystick */
  				switch(event.xbutton.button) {
 					case Button1 :	ski_modifier_reg &= ~0x80;
