@@ -22,6 +22,7 @@
 #include <stdlib.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
+#include <X11/Xatom.h>
 #ifdef MITSHM
 #include <sys/ipc.h>
 #include <sys/shm.h>
@@ -35,8 +36,8 @@ Screen		*vid_screen;
 Display		*vid_display;
 Visual		*vid_visual;
 XImage		*vid_image;
-Window		vid_root,vid_main_window,vid_window;
-GC		vid_gc;
+Window		vid_root,vid_main_window,vid_window,vid_stat_window;
+GC		vid_gc,vid_stat_gc;
 Colormap	vid_colormap;
 Atom		wm_delete;
 XFontStruct	*vid_fontinfo;
@@ -44,11 +45,12 @@ XFontStruct	*vid_fontinfo;
 XShmSegmentInfo	vid_shminfo;
 #endif
 
-PIXEL		*vid_lines[VID_WIN_HEIGHT];
-PIXEL		*vid_buffer;
+void			*vid_lines[VID_WIN_HEIGHT];
+void			*vid_buffer;
 
-int		vid_shm;
-int		vid_mapped;
+int			vid_shm;
+int			vid_mapped;
+int			vid_class;
 
 XColor	vid_supercolors[256];
 XColor	vid_textcolors[16] = {
@@ -118,16 +120,16 @@ int VID_checkForSHM(Display *display)
 int VID_allocShmImage(void)
 {
 #ifdef MITSHM
+
    /*
     * We have to do a _lot_ of error checking here to be sure we can
     * really use the XSHM extension.  It seems different servers trigger
     * errors at different points if the extension won't work.  Therefore
     * we have to be very careful...
     */
-	GC	gc;
 	int	(*old_handler)(Display *, XErrorEvent *);
 
-	vid_image = XShmCreateImage(vid_display, vid_visual, VID_DEPTH,
+	vid_image = XShmCreateImage(vid_display, vid_visual, vid_depth,
 					ZPixmap, NULL, &vid_shminfo,
 					VID_WIDTH, VID_HEIGHT);
 
@@ -137,7 +139,7 @@ int VID_allocShmImage(void)
 	}
 
 	vid_shminfo.shmid = shmget(IPC_PRIVATE, vid_image->bytes_per_line
-					*vid_image->height, IPC_CREAT|0777);
+					* vid_image->height, IPC_CREAT|0777);
 	if (vid_shminfo.shmid < 0) {
 		perror("VID_allocShmImage");
 		XDestroyImage(vid_image);
@@ -182,11 +184,9 @@ int VID_allocShmImage(void)
 
    /* Finally, try an XShmPutImage to be really sure the extension works */
 
-	gc = XCreateGC(vid_display, vid_window, 0, NULL);
-	XShmPutImage(vid_display, vid_window, gc,
+	XShmPutImage(vid_display, vid_window, vid_gc,
 			vid_image, 0, 0, 0, 0, 1, 1 /*one pixel*/, False);
 	XSync(vid_display, False);
-	XFreeGC(vid_display, gc);
 	XSetErrorHandler(old_handler);
 	if (vid_xerrorflag) {
 		XFlush(vid_display);
@@ -199,7 +199,7 @@ int VID_allocShmImage(void)
 		return 0;
 	}
 
-	vid_buffer = (PIXEL *) vid_shminfo.shmaddr;
+	vid_buffer = (void *) vid_shminfo.shmaddr;
 	return 1;
 #else
 	return 0;
@@ -215,17 +215,12 @@ int VID_outputInit(void)
 	XVisualInfo	*visualList;
 	int		visualsMatched;
 	int		visual_chosen;
-	XSetWindowAttributes win_attr;
 	XGCValues	gcvals;
 
 	int		i;
-
-	printf("    - Initializing XThreads: ");
-	if (!XInitThreads()) {
-		printf("Failed\n");
-		return 1;
-	}
-	printf("Done\n");
+	int		ret;
+	unsigned long	m;
+	XSetWindowAttributes	attr;
 
 	printf("    - Opening display: ");
 	vid_display = XOpenDisplay(NULL);
@@ -234,27 +229,26 @@ int VID_outputInit(void)
 		return 1;
 	}
 	vid_screen = DefaultScreenOfDisplay(vid_display);
-/*
-	vid_black = BlackPixelOfScreen(vid_screen);
-	vid_white = WhitePixelOfScreen(vid_screen);
-*/
-	vid_black = 0;
-	vid_white = 15;
 	printf("Done\n");
 
 	printf("    - Picking a visual: ");
+	if (vid_depth == 8) {
+		vid_class = PseudoColor;
+	} else {
+		vid_class = TrueColor;
+	}
 	vTemplate.screen = DefaultScreen(vid_display);
-	vTemplate.depth = VID_DEPTH;
+	vTemplate.depth = vid_depth;
 	visualList = XGetVisualInfo(vid_display,
 		VisualScreenMask | VisualDepthMask,
 		&vTemplate, &visualsMatched);
-	if(visualsMatched == 0) {
+	if (visualsMatched == 0) {
 		fprintf(stderr, "no visuals!\n");
 		return 1;
 	}
 	visual_chosen = -1;
 	for(i = 0; i < visualsMatched; i++) {
-		if (visualList[i].class == VID_CLASS) {
+		if (visualList[i].class == vid_class) {
 			visual_chosen = i;
 			break;
 		}
@@ -269,15 +263,22 @@ int VID_outputInit(void)
 
 	printf("    - Creating main window: ");
 	vid_root = RootWindowOfScreen(vid_screen);
-	vid_colormap = XCreateColormap(vid_display, vid_root, vid_visual, AllocAll);
-	win_attr.border_pixel = 255;
-	win_attr.background_pixel = 255;
-	win_attr.colormap = vid_colormap;
 	vid_mapped = 0;
+	if (vid_class == PseudoColor) {
+		vid_colormap = XCreateColormap(vid_display, vid_root,
+						vid_visual, AllocAll);
+	} else {
+		vid_colormap = XCreateColormap(vid_display, vid_root,
+						vid_visual, AllocNone);
+	}
+	attr.colormap = vid_colormap;
+	attr.background_pixel = 255;
+	attr.border_pixel = BlackPixelOfScreen(vid_screen);
 	vid_main_window = XCreateWindow(vid_display, vid_root, 0, 0,
-				VID_WIN_WIDTH,VID_WIN_HEIGHT, 0, vTemplate.depth, InputOutput,
-				vid_visual, CWColormap | CWBackPixel | CWBorderPixel,
-				&win_attr);
+				VID_WIN_WIDTH, VID_WIN_HEIGHT, 4,
+				vid_depth, InputOutput, vid_visual,
+				CWColormap | CWBackPixel | CWBorderPixel,
+				&attr);
 	if (!vid_main_window) {
 		printf("Failed\n");
 		return 1;
@@ -285,21 +286,34 @@ int VID_outputInit(void)
 	printf("Done\n");
 
 	printf("    - Creating display subwindow: ");
-	win_attr.border_pixel = vid_white;
-	win_attr.background_pixel = vid_black;
-	win_attr.colormap = vid_colormap;
-	vid_window = XCreateWindow(vid_display, vid_main_window,
-			(VID_WIN_WIDTH - VID_WIDTH) / 2, (VID_WIN_HEIGHT - VID_HEIGHT) / 2,
-			VID_WIDTH, VID_HEIGHT, 0, vTemplate.depth, InputOutput,
-			vid_visual, CWColormap | CWBackPixel | CWBorderPixel,
-			&win_attr);
+	vid_window = XCreateSimpleWindow(vid_display, vid_main_window,
+			(VID_WIN_WIDTH - VID_WIDTH) / 2,
+			(VID_WIN_HEIGHT - VID_HEIGHT) / 2,
+			VID_WIDTH, VID_HEIGHT, 0,
+			BlackPixelOfScreen(vid_screen),
+			WhitePixelOfScreen(vid_screen));
 	if (!vid_window) {
 		printf("Failed\n");
 		return 1;
 	}
+	vid_gc = XCreateGC(vid_display, vid_window, 0, NULL);
 	printf("Done\n");
 
-	XStoreName(vid_display, vid_main_window, VERSION);
+	printf("    - Creating status/control window: ");
+	vid_stat_window = XCreateSimpleWindow(vid_display, vid_root, 0, 0,
+				VID_STAT_WIDTH,VID_STAT_HEIGHT, 4,
+				BlackPixelOfScreen(vid_screen),
+				WhitePixelOfScreen(vid_screen));
+	if (!vid_stat_window) {
+		printf("Failed\n");
+		return 1;
+	}
+	vid_stat_gc = XCreateGC(vid_display, vid_stat_window, 0, NULL);
+	printf("Done\n");
+
+	XStoreName(vid_display, vid_main_window, "XGS Display");
+	XStoreName(vid_display, vid_stat_window, "XGS Controls");
+
 	size_hints.flags = PSize | PMinSize | PMaxSize;
 	size_hints.base_width = VID_WIN_WIDTH;
 	size_hints.min_width = VID_WIN_WIDTH;
@@ -311,48 +325,57 @@ int VID_outputInit(void)
 	wm_hints.flags = InputHint;
 	XSetWMNormalHints(vid_display, vid_main_window, &size_hints);
 	XSetWMHints(vid_display, vid_main_window, &wm_hints);
-	/* We will handle WM_DELETE window manager messages */
 	wm_delete = XInternAtom(vid_display,"WM_DELETE_WINDOW",True);
-	if(wm_delete)
-	  XSetWMProtocols(vid_display, vid_main_window, &wm_delete, 1);
+	if(wm_delete) {
+		XSetWMProtocols(vid_display, vid_main_window, &wm_delete, 1);
+	}
 	XSelectInput(vid_display,vid_main_window,ExposureMask|StructureNotifyMask);
 	XSelectInput(vid_display,vid_window,ExposureMask|KeyPressMask|KeyReleaseMask|ButtonPressMask|ButtonReleaseMask|PointerMotionMask);
+
+	XMapRaised(vid_display, vid_stat_window);
 	XMapRaised(vid_display, vid_main_window);
 	XMapRaised(vid_display, vid_window);
 
 	printf("    - Setting status font: ");
-	vid_gc = XCreateGC(vid_display, vid_main_window, 0, NULL);
 	vid_fontinfo = XLoadQueryFont(vid_display,VID_STATUS_FONT);
 	gcvals.font = vid_fontinfo->fid;
 	gcvals.fill_style = FillSolid;
-	XChangeGC(vid_display, vid_gc, GCFillStyle | GCFont, &gcvals);
-	XSetForeground(vid_display, vid_gc, vid_white);
-	XSetBackground(vid_display, vid_gc, vid_black);
+	XChangeGC(vid_display, vid_stat_gc, GCFillStyle | GCFont, &gcvals);
+	XSetForeground(vid_display, vid_stat_gc,
+			BlackPixelOfScreen(vid_screen));
+	XSetBackground(vid_display, vid_stat_gc,
+			WhitePixelOfScreen(vid_screen));
 	printf("Done\n");
 
+	if (vid_depth == 24) {
+		vid_pixsize = 4;
+	} else {
+		vid_pixsize = vid_depth / 8;
+	}
+
 #ifdef MITSHM
 	printf("    - Checking for MIT-SHM support: ");
 	vid_shm = VID_checkForSHM(vid_display);
 	if (vid_shm) {
-		printf("Done\n");
+		printf("Found\n");
 	} else {
-		printf("Failed\n");
+		printf("Not Found\n");
 	}
 	printf("    - Creating shared image: ");
 	if (VID_allocShmImage()) {
 		printf("Done\n");
 	} else {
-		printf("Failed! SHM disabled.\n");
+		printf("Failed.  SHM disabled.\n");
 	}
 #else
 	vid_shm = 0;
 #endif
 	if (!vid_shm) {
 		printf("    - Creating Image: ");
-		vid_image = XCreateImage(vid_display, vid_visual, VID_DEPTH,
-						ZPixmap, 0,   /* format, offset */
-						NULL, VID_WIDTH, VID_HEIGHT,
-						sizeof(PIXEL)*8, 0);  /* pad, bytes_per_line */
+		vid_image = XCreateImage(vid_display, vid_visual, vid_depth,
+						ZPixmap, 0, NULL,
+						VID_WIDTH, VID_HEIGHT, 
+						vid_pixsize * 8, 0);
 		if (!vid_image) {
 			printf("Failed\n");
 			return 1;
@@ -370,8 +393,8 @@ int VID_outputInit(void)
 	}
 
 	vid_lines[0] = vid_buffer;
-	for (i = 1 ; i < VID_HEIGHT ; i++) {
-		vid_lines[i] = vid_lines[i-1] + VID_WIDTH;
+	for (i = 1 ; i < vid_image->height ; i++) {
+		vid_lines[i] = vid_lines[i-1] + vid_image->bytes_per_line;
 	}
 
 	return 0;
@@ -416,7 +439,9 @@ void VID_outputShutdown()
 	    vid_image = NULL;
 	}
 	printf("    - Closing display\n");
-	XFreeColormap(vid_display, vid_colormap);
+	if (vid_depth == 8) {
+		XFreeColormap(vid_display, vid_colormap);
+	}
 	XCloseDisplay(vid_display);
 }
 
@@ -427,7 +452,7 @@ void VID_outputStatus1(char *msg)
 	height = vid_fontinfo->ascent + vid_fontinfo->descent;
 	margin = vid_fontinfo->ascent;
 
-	XDrawImageString(vid_display, vid_main_window, vid_gc, 0, VID_WIN_HEIGHT-height*2-margin, msg, strlen(msg));
+	XDrawImageString(vid_display, vid_stat_window, vid_stat_gc, 0, 0, msg, strlen(msg));
 }
 
 void VID_outputStatus2(char *msg)
@@ -437,7 +462,7 @@ void VID_outputStatus2(char *msg)
 	height = vid_fontinfo->ascent + vid_fontinfo->descent;
 	margin = vid_fontinfo->ascent;
 
-	XDrawImageString(vid_display, vid_main_window, vid_gc, 0, VID_WIN_HEIGHT-height-margin, msg, strlen(msg));
+	XDrawImageString(vid_display, vid_stat_window, vid_stat_gc, 0, height+margin, msg, strlen(msg));
 }
 
 void VID_outputResize(int width, int height)
@@ -451,34 +476,45 @@ void VID_outputResize(int width, int height)
 
 void VID_outputSetTextColors()
 {
-	vid_textcolors[vid_textbgcolor].pixel = 253;
-	XStoreColor(vid_display, vid_colormap, &vid_textcolors[vid_textbgcolor]);
-	vid_textcolors[vid_textfgcolor].pixel = 254;
-	XStoreColor(vid_display, vid_colormap, &vid_textcolors[vid_textfgcolor]);
+	int		i;
+
+	if (vid_depth == 8) {
+		vid_textcolors[vid_textbgcolor].pixel = 253;
+		XStoreColor(vid_display, vid_colormap,
+				&vid_textcolors[vid_textbgcolor]);
+		vid_textcolors[vid_textfgcolor].pixel = 254;
+		XStoreColor(vid_display, vid_colormap,
+				&vid_textcolors[vid_textfgcolor]);
+		vid_textcolors[vid_bordercolor].pixel = 255;
+		XStoreColor(vid_display, vid_colormap,
+				&vid_textcolors[vid_bordercolor]);
+		for (i = 0 ; i <= 15 ; i++) vid_textcolors[i].pixel = i;
+		XStoreColors(vid_display, vid_colormap, vid_textcolors, 16);
+	} else {
+		vid_textpixel[0] = VID_outputGetPixel(vid_textbgcolor);
+		vid_textpixel[1] = VID_outputGetPixel(vid_textfgcolor);
+		vid_borderpixel = VID_outputGetPixel(vid_bordercolor);
+		for (i = 0 ; i <= 15 ; i++) {
+			vid_grpixel[i] = VID_outputGetPixel(i);
+		}
+		XSetWindowBackground(vid_display, vid_main_window,
+					vid_borderpixel);
+		XClearWindow(vid_display, vid_main_window);
+	}
 }
 
 void VID_outputSetBorderColor()
 {
-	vid_textcolors[vid_bordercolor].pixel = 255;
-	XStoreColor(vid_display, vid_colormap, &vid_textcolors[vid_bordercolor]);
-}
-
-void VID_outputSetStandardColors()
-{
-	int	i;
-
-	vid_textcolors[vid_textbgcolor].pixel = 253;
-	XStoreColor(vid_display, vid_colormap, &vid_textcolors[vid_textbgcolor]);
-	vid_textcolors[vid_textfgcolor].pixel = 254;
-	XStoreColor(vid_display, vid_colormap, &vid_textcolors[vid_textfgcolor]);
-	vid_textcolors[vid_bordercolor].pixel = 255;
-	XStoreColor(vid_display, vid_colormap, &vid_textcolors[vid_bordercolor]);
-	for (i = 0 ; i <= 15 ; i++) {
-		vid_textcolors[i].pixel = i;
+	if (vid_depth == 8) {
+		vid_textcolors[vid_bordercolor].pixel = 255;
+		XStoreColor(vid_display, vid_colormap,
+				&vid_textcolors[vid_bordercolor]);
+	} else {
+		vid_borderpixel = VID_outputGetPixel(vid_bordercolor);
+		XSetWindowBackground(vid_display, vid_main_window,
+					vid_borderpixel);
+		XClearWindow(vid_display, vid_main_window);
 	}
-	XStoreColors(vid_display, vid_colormap, vid_textcolors, 16);
-	vid_black = 0;
-	vid_white = 15;
 }
 
 void VID_outputSetSuperColors()
@@ -486,17 +522,49 @@ void VID_outputSetSuperColors()
 	int	addr,i;
 
 	addr = 0x019E00;
-	for (i = 0 ; i < 256 ; i++) {
-		if (mem_slowram_changed[addr >> 8] & mem_change_masks[addr & 0xFF]) {
-			vid_supercolors[i].blue = (slow_memory[addr] & 0x0F) * 4369;
-			vid_supercolors[i].green = ((slow_memory[addr] >> 4) & 0x0F) * 4369;
-			vid_supercolors[i].red = (slow_memory[addr+1] & 0x0F) * 4369;
-			vid_supercolors[i].flags = DoRed | DoGreen | DoBlue;
-			vid_supercolors[i].pixel = i;
-			XStoreColor(vid_display, vid_colormap, &vid_supercolors[i]);
+	if (vid_depth == 8) {
+		for (i = 0 ; i < 256 ; i++) {
+			if (mem_slowram_changed[addr >> 8] & mem_change_masks[addr & 0xFF]) {
+				vid_supercolors[i].blue = (slow_memory[addr] & 0x0F) * 4369;
+				vid_supercolors[i].green = ((slow_memory[addr] >> 4) & 0x0F) * 4369;
+				vid_supercolors[i].red = (slow_memory[addr+1] & 0x0F) * 4369;
+				vid_supercolors[i].flags = DoRed | DoGreen | DoBlue;
+				vid_supercolors[i].pixel = i;
+				XStoreColor(vid_display, vid_colormap, &vid_supercolors[i]);
+			}
+			addr += 2;
+		}
+	} else {
+		for (i = 0 ; i < 256 ; i++) {
+			if (mem_slowram_changed[addr >> 8] & mem_change_masks[addr & 0xFF]) {
+				vid_supercolors[i].blue = (slow_memory[addr] & 0x0F) * 4369;
+				vid_supercolors[i].green = ((slow_memory[addr] >> 4) & 0x0F) * 4369;
+				vid_supercolors[i].red = (slow_memory[addr+1] & 0x0F) * 4369;
+				vid_supercolors[i].flags = DoRed | DoGreen | DoBlue;
+				vid_supercolors[i].pixel = i;
+				XAllocColor(vid_display, vid_colormap, &vid_supercolors[i]);
+				vid_shrpixel[i] = vid_supercolors[i].pixel;
+			}
+			addr += 2;
 		}
-		addr += 2;
 	}
 	mem_slowram_changed[0x019E] = 0;
 	mem_slowram_changed[0x019F] = 0;
 }
+
+unsigned long VID_outputGetPixel(int colornum)
+{
+	XAllocColor(vid_display, vid_colormap, &vid_textcolors[colornum]);
+	return vid_textcolors[colornum].pixel;
+}
+
+unsigned long VID_outputRGB2Pixel(int red, int green, int blue)
+{
+	XColor	color;
+
+	color.red = red;
+	color.green = green;
+	color.blue = blue;
+	XAllocColor(vid_display, vid_colormap, &color);
+	return color.pixel;
+}
