@@ -38,6 +38,8 @@
 #include "vid-drv.h"
 #include "cpu.h"
 
+int	emul_period;
+
 int	emul_vbl;
 int	emul_vblirq;
 int	emul_qtrsecirq;
@@ -46,6 +48,7 @@ int	emul_scanirq;
 
 int	emul_speed;
 int	emul_speed2;
+int	emul_delay;
 
 int	emul_tick;
 int	emul_microtick;
@@ -74,31 +77,38 @@ char	*emul_s5d1,*emul_s5d2;
 char	*emul_s6d1,*emul_s6d2;
 char	*emul_smtport[NUM_SMPT_DEVS];
 
-pthread_t	emul_cpu_thread;
-pthread_t	emul_snd_thread;
-pthread_t	emul_adb_thread;
-pthread_t	emul_vid_thread;
-
 void EMUL_doVBL()
 {
 	int	cycles;
 	double	this_time,last_time,diff,speed;
+	int	new_period;
+
+#ifdef HAVE_GETTIMEOFDAY
+	if (emul_delay > 0) {
+#ifdef EMX_HAVE_SLEEP2        
+		_sleep2(emul_delay);
+#else
+//		usleep(emul_delay);
+#endif  // end of #ifdef EMX_HAVE_SLEEP2
+	}
+#endif  // end of #ifdef HAVE_GETTIMEOFDAY
 
 	cycles = cpu_cycle_count - emul_last_cycles;
 	emul_last_cycles = cpu_cycle_count;
 
 	emul_vbl_count++;
 
-	if (emul_vblirq) {
-		if (!(mem_diagtype & 0x08)) {
-			mem_diagtype |= 0x08;
-			CPU_addIRQ();
+	if (emul_vbl == 0x00) {
+		if (emul_vblirq) {
+			if (!(mem_diagtype & 0x08)) {
+				mem_diagtype |= 0x08;
+				CPU_addIRQ();
+			}
 		}
+		VID_update();
+		ADB_update();
+		IWM_update();
 	}
-	MEM_update();
-	CLK_update();
-	IWM_update();
-	SMPT_update();
 
 	if ((emul_tick == 0) || (emul_tick == 15) || (emul_tick == 30) || (emul_tick == 45)) {
 		if (emul_qtrsecirq) {
@@ -135,8 +145,21 @@ void EMUL_doVBL()
 
 	speed = (emul_total_cycles / emul_total_time)  / 1000;
 
+	new_period = speed * 32.0;
+
+	if (new_period < 15) new_period = 15;
+
+	emul_period = new_period;
+	CPU_setUpdatePeriod(new_period);
+
+	if ((speed > emul_target_speed) && emul_target_cycles) {
+		emul_delay = ((emul_total_cycles - emul_target_cycles) * 1000000) / emul_total_cycles;
+	} else {
+		emul_delay = 0;
+	}
+
 	if (!emul_tick) {
-		sprintf(emul_buffer,"Target Speed: %2.1f MHz  Actual Speed: %2.1f MHz", emul_target_speed, speed);
+		sprintf(emul_buffer,"Target Speed: %2.1f MHz  Actual Speed: %2.1f MHz  Period = %3d cycles", emul_target_speed, speed,new_period);
 		VID_outputStatus2(emul_buffer);
 	}
 
@@ -145,7 +168,7 @@ void EMUL_doVBL()
 
 void EMUL_hardwareUpdate(word32 cycles)
 {
-loop:
+	if (snd_enable) SND_update();
 	emul_microtick++;
 	if (emul_microtick == 400) {
 		emul_vbl = 0x80;
@@ -155,7 +178,6 @@ loop:
 		vid_vert_cnt = 0;
 		emul_vbl = 0x00;
 		EMUL_doVBL();
-		usleep(16000);
 	} else {
 		vid_vert_cnt = emul_microtick >> 1;
 	}
@@ -166,8 +188,6 @@ loop:
 			CPU_addIRQ();
 		}
 	}
-	sched_yield();
-	goto loop;
 }
 
 int EMUL_init(int argc, char *argv[])
@@ -189,6 +209,13 @@ int EMUL_init(int argc, char *argv[])
 			EMUL_trace(1);
 		} else if (!strcmp(argv[i],"-nosound")) {
 			snd_enable = 0;
+		} else if (!strcmp(argv[i],"-window")) {
+			vid_fullscreen = 0;
+		} else if (!strcmp(argv[i],"-fullscreen")) {
+			vid_fullscreen = 1;
+		} else if (!strcmp(argv[i],"-depth")) {
+			if (++i >= argc) EMUL_displayUsage(argv[0]);
+			vid_depth = atoi(argv[i]);
 		} else if (!strcmp(argv[i],"-ram")) {
 			if (++i >= argc) EMUL_displayUsage(argv[0]);
 			mem_ramsize = atoi(argv[i]);
@@ -344,38 +371,17 @@ void EMUL_run()
 			printf("Done\n");
 		}
 	}
-	printf("\nSpawning threads:\n");
-
-	if (pthread_create(&emul_cpu_thread, NULL, (void *) CPU_run, NULL)) {
-		perror("Couldn't spawn thread");
-		exit(-1);
-	}
-	printf("    - CPU\n");
-	if (pthread_create(&emul_vid_thread, NULL, (void *) VID_run, NULL)) {
-		perror("Couldn't spawn thread");
-		exit(-1);
-	}
-	printf("    - Video\n");
-
-	if (pthread_create(&emul_adb_thread, NULL, (void *) ADB_run, NULL)) {
-		perror("Couldn't spawn thread");
-		exit(-1);
-	}
-	printf("    - ADB\n");
-	if (snd_enable) {
-		if (pthread_create(&emul_snd_thread, NULL, (void *) SND_run, NULL)) {
-			perror("Couldn't spawn thread");
-			exit(-1);
-		}
-		printf("    - Sound\n");
-	}
-
 	printf("\n*** EMULATOR IS RUNNING ***\n");
-	EMUL_hardwareUpdate(0);
+
+	emul_period = 64;
+	CPU_setUpdatePeriod(64);
+	CPU_run();
 }
 
 void EMUL_reset()
 {
+	emul_delay = 0;
+
 	MEM_reset();
 	VID_reset();
 	SND_reset();
@@ -446,7 +452,7 @@ void EMUL_displayUsage(char *myname)
 #include <time.h>
 #include <sys/time.h>
 
-void	usleep( unsigned int microSeconds )
+int     usleep( unsigned int microSeconds )
 {
 	unsigned int	    Seconds, uSec;
 	int		     nfds, readfds, writefds, exceptfds;
@@ -457,8 +463,9 @@ void	usleep( unsigned int microSeconds )
 	if( (microSeconds == (unsigned long) 0) 
 		|| microSeconds > (unsigned long) 4000000 )
 	{
+		errno = ERANGE;	 /* value out of range */
 		perror( "usleep time out of range ( 0 -> 4000000 ) " );
-		return;
+		return -1;
 	}
 
 	Seconds = microSeconds / (unsigned long) 1000000;
@@ -470,7 +477,9 @@ void	usleep( unsigned int microSeconds )
 	if( select( nfds, &readfds, &writefds, &exceptfds, &Timer ) < 0 )
 	{
 		perror( "usleep (select) failed" );
-		return;
+		return -1;
 	}
+
+	return 0;
 }
 #endif
