@@ -1,11 +1,10 @@
 #define VID_WIDTH	640
 #define VID_HEIGHT	400
 
-#define VID_STATUS_FONT	"8x13"
+#define VID_DEFAULT_DEPTH	8
+#define VID_DEFAULT_FULLSCREEN	1
 
-#define PIXEL byte
-#define VID_DEPTH 8
-#define VID_CLASS PseudoColor
+#define VID_STATUS_FONT	"8x13"
 
 #define VID_MODE_40TEXT1	0
 #define VID_MODE_40TEXT2	1
@@ -24,7 +23,8 @@
 
 #define VID_MODE_SUPER		12
 
-extern PIXEL	vid_black,vid_white;
+extern int	vid_depth,vid_pixsize,vid_fullscreen;
+
 extern byte	*vid_font40[2];
 extern byte	*vid_font80[2];
 extern int	vid_currmode;
@@ -37,6 +37,11 @@ extern int	vid_bordercolor;
 extern int	vid_textfgcolor;
 extern int	vid_textbgcolor;
 
+extern unsigned long	vid_borderpixel;
+extern unsigned long	vid_textpixel[2];
+extern unsigned long	vid_grpixel[16];
+extern unsigned long	vid_shrpixel[256];
+
 extern int	vid_80col;
 extern int	vid_altcharset;
 
@@ -112,3 +117,6 @@ void VID_refreshDHiresPage1(void);
 void VID_refreshDHiresPage2(void);
 
 void VID_refreshSuperHires(void);
+void VID_refreshSuperHires8(void);
+void VID_refreshSuperHires16(void);
+void VID_refreshSuperHires32(void);
