@@ -43,8 +43,6 @@ static int	sr,num_osc;
 
 static int	snd_click_sample;
 
-pthread_mutex_t	snd_mutex = PTHREAD_MUTEX_INITIALIZER;
-
 const static int	wp_masks[8] = { 0xFF, 0xFE, 0xFC, 0xF8, 0xF0, 0xE0, 0xC0, 0x80 };
 const static int	acc_masks[8] = { 0x00FF, 0x01FF, 0x03FF, 0x07FF, 0x0FFF, 0x1FFF, 0x3FFF, 0x7FFF };
 
@@ -80,14 +78,6 @@ static int	output_index;
 static int	irq_stack[IRQ_STACK_SIZE];
 static int	irq_index;
 
-void *SND_run(void *parm)
-{
-loop:
-	SND_update();
-	sched_yield();
-	goto loop;
-}
-
 int SND_init()
 {
 	int	i;
@@ -123,11 +113,8 @@ void SND_update()
 #if 0
 	static int noisy = 0;
 #endif
-	if (!snd_enable) return;
 	if (output_index == OUTPUT_BUFFER_SIZE) {
-		if (!SND_outputWrite(output_buffer,output_index)) {
-			return;
-		}
+		if (!SND_outputWrite(output_buffer,output_index)) return;
 		output_index = 0;
 #if 0
 		size_t nsamples = SND_outputWrite(output_buffer,output_index);
@@ -150,9 +137,7 @@ void SND_update()
 		snd_click_sample = 0;
 #endif
 	}
-	pthread_mutex_lock(&snd_mutex);
 	SND_scanOscillators(&sample);
-	pthread_mutex_unlock(&snd_mutex);
 
 	output_buffer[output_index].left = sample.left + snd_click_sample;
 	output_buffer[output_index].right = sample.right + snd_click_sample;
@@ -175,7 +160,6 @@ void SND_reset()
 {
 	int	i;
 
-	pthread_mutex_lock(&snd_mutex);
 	snd_click_sample = 0;
 
 	for (i = 0 ; i < 32 ; i++) {
@@ -197,7 +181,6 @@ void SND_reset()
 	output_index = 0;
 
 	SND_enableOscillators();
-	pthread_mutex_unlock(&snd_mutex);
 }
 
 void SND_shutdown()
@@ -221,7 +204,6 @@ byte SND_readSoundData(byte val)
 {
 	int	reg,osc_num;
 
-	pthread_mutex_lock(&snd_mutex);
 	val = glu_next_val;
 	if (glu_ctrl_reg & 0x40) {
 		glu_next_val = doc_ram[glu_addr_reg & 0xFFFF];
@@ -238,7 +220,6 @@ byte SND_readSoundData(byte val)
 		glu_next_val = doc_registers[reg];
 	}
 	if (glu_ctrl_reg & 0x20) glu_addr_reg++;
-	pthread_mutex_unlock(&snd_mutex);
 	return val;
 }
 
@@ -262,7 +243,6 @@ byte SND_writeSoundData(byte val)
 {
 	int	reg;
 
-	pthread_mutex_lock(&snd_mutex);
 	if (glu_ctrl_reg & 0x40) {
 		doc_ram[glu_addr_reg & 0xFFFF] = val;
 	} else {
@@ -280,23 +260,18 @@ byte SND_writeSoundData(byte val)
 		if (reg < 0xE0) SND_updateOscillator(reg & 0x1F);
 	}
 	if (glu_ctrl_reg & 0x20) glu_addr_reg++;
-	pthread_mutex_unlock(&snd_mutex);
 	return 0;
 }
 
 byte SND_writeSoundAddrL(byte val)
 {
-	pthread_mutex_lock(&snd_mutex);
 	glu_addr_reg = (glu_addr_reg & 0xFF00) | val;
-	pthread_mutex_unlock(&snd_mutex);
 	return 0;
 }
 
 byte SND_writeSoundAddrH(byte val)
 {
-	pthread_mutex_lock(&snd_mutex);
 	glu_addr_reg = (val << 8) | (glu_addr_reg & 0x00FF);
-	pthread_mutex_unlock(&snd_mutex);
 	return 0;
 }
 
