@@ -25,19 +25,26 @@
 #include "video.h"
 #include "vid-drv.h"
 
-PIXEL		vid_black, vid_white;
-
 byte		*vid_font40[2];
 byte		*vid_font80[2];
 
 void		(*VID_updateRoutine)(void);
 
+int		vid_depth = VID_DEFAULT_DEPTH;
+int		vid_pixsize = 1;
+int		vid_fullscreen = VID_DEFAULT_FULLSCREEN;
+
 int		vid_xmin, vid_ymin, vid_xmax, vid_ymax;
 
 int		vid_bordercolor;
 int		vid_textfgcolor;
 int		vid_textbgcolor;
 
+unsigned long	vid_borderpixel;
+unsigned long	vid_textpixel[2];
+unsigned long	vid_grpixel[16];
+unsigned long	vid_shrpixel[256];
+
 int		vid_currmode;
 
 int		vid_super;
@@ -59,8 +66,6 @@ int		vid_horiz_cnt;
 
 int		vid_need_redraw;
 
-pthread_mutex_t	vid_mutex = PTHREAD_MUTEX_INITIALIZER;
-
 int VID_init(void)
 {
 	char	*path;
